﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;

namespace GenericLibrary
{
    public class CatalogDictionary<TId, TElement> : ICatalog<TId, TElement> where TElement : IIdentifiable<TId>
    {
        private readonly IDictionary<TId, TElement> _elements = new Dictionary<TId, TElement>();
        public IEnumerator<TElement> GetEnumerator()
        {
            return _elements.Values.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        public bool Add(TElement element)
        {
            TId id = element.Id;
            if (_elements.ContainsKey(id)) { return false; }
            _elements[id] = element;
            return true;
        }

        public int Count { get { return _elements.Count; } }

        public TElement Find(TId id)
        {
            if (_elements.ContainsKey(id))
                return _elements[id];
            return default(TElement);
        }

        public IEnumerable<TElement> GetAll()
        {
            // HACK should return read-only copy
            return _elements.Values;
        }

        public IReadOnlyCollection<TElement> GetAll2()
        {
            return new ReadOnlyCollection<TElement>(_elements.Values.ToList());
        } 

        public IEnumerable<TElement> GetAll(Predicate<TElement> predicate)
        {
            IList<TElement> results = new List<TElement>();
            foreach (TElement element in _elements.Values)
            {
                if (predicate.Invoke(element))
                {
                    results.Add(element);
                }
            }
            return results;
        }
    }
}
