﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;

namespace GenericLibrary
{
    class CatalogSet<TId, TElement> : ICatalog<TId, TElement> where TElement : IIdentifiable<TId>
    {
        private readonly ISet<TElement> _elements = new HashSet<TElement>();

        //public CatalogSet(IEnumerable<TElement> ) 

        public IEnumerator<TElement> GetEnumerator()
        {
            return _elements.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        public bool Add(TElement element)
        {
            return _elements.Add(element);
        }

        public int Count { get; private set; }
        public TElement Find(TId id)
        {
            foreach (TElement element in _elements)
            {
                if (id.Equals(element.Id))
                {
                    return element;
                }
            }
            return default(TElement);
        }

        public IEnumerable<TElement> GetAll()
        {
            return _elements;
        }

        public IReadOnlyCollection<TElement> GetAll2()
        {
            return new ReadOnlyCollection<TElement>(_elements.ToList());
        } 

        public IEnumerable<TElement> GetAll(Predicate<TElement> predicate)
        {
            IList<TElement> results = new List<TElement>();
            foreach (TElement element in _elements)
            {
                if (predicate.Invoke(element))
                {
                    results.Add(element);
                }
            }
            return results;
        }
    }
}
