﻿using System;
using System.Collections.Generic;
using System.Linq;
using GenericLibrary;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace GenericCatalogTests
{
    [TestClass]
    public class CatalogDictionaryTest
    {
        private readonly Book _book1 = new Book(1, "Anders", "C#");
        private readonly Book _book2 = new Book(2, "John Sharp", "C# 2012");
        private ICatalog<int, Book> _books;

        [TestInitialize]
        public void Intialize()
        {
            _books = new CatalogDictionary<int, Book>();
            _books.Add(_book1);
            _books.Add(_book2);
        }

        [TestMethod()]
        public void AddCountTest()
        {
            Assert.AreEqual(2, _books.Count);
            Book anotherBook = new Book(3, "Bard de Smet", "C# 5.0 Unleashed");
            bool added = _books.Add(anotherBook);
            Assert.IsTrue(added);
            Assert.AreEqual(3, _books.Count);
        }

        [TestMethod()]
        public void FindTest()
        {
            Book b = _books.Find(1);
            Assert.AreEqual(b, _book1);
            Assert.IsNull(_books.Find(100));
        }

        [TestMethod()]
        public void GetAllTest()
        {
            IEnumerable<Book> allBooks = _books.GetAll();
            Assert.AreEqual(2, allBooks.Count());

            IEnumerable<Book> someBooks = _books.GetAll(book => book.Author.StartsWith("A"));
            Assert.AreEqual(1, someBooks.Count());
        }
    }

    
}
